#include "hal_cc2530znp_target_board.h"

#define HARDWARE_UART
/**Comment this out to reduce code size (not including math.h)*/
#define Convert_to_lux

/** Function pointer for the ISR called when a timer generates an interrupt*/
void (*timerIsr)(void);


/** Flags to indicate when to wake up the processor. These are read in the various interrupt service routines 
and if the flag is set then the processor will be woken up with HAL_WAKEUP() at the end of the ISR. 
This is required because HAL_WAKEUP() cannot be called anywhere except in an ISR. */
unsigned int wakeupFlags = 0;

/** The post-calibrated frequency of the Very Low Oscillator (VLO). Set with calibrateVlo() and read by initTimer(). */
unsigned int vloFrequency = 0;

/**Required for the printf command*/
static char* bf;
static char buf[12];
static unsigned int num;
static char uc;
static char zs;

/** 
Configures hardware for the particular hardware platform:
- Ports: sets direction, interrupts, pullup/pulldown resistors etc. 
- Oscillator: turns off WDT, configures MCLK = 8MHz using internal DCO & sets SMCLK = 4MHz
- Holds radio in reset (active-low)
*/
void halInit()
{
    WDTCTL = WDTPW + WDTHOLD; // Stop WDT
    
    //
    //    Initialize Oscillator
    //
    if (CALBC1_8MHZ ==0xFF || CALDCO_8MHZ == 0xFF)                                     
    {  
        while(1); // Stop if calibration constants erased
    }   
    BCSCTL1 = CALBC1_8MHZ; // Set DCO = 8MHz for MCLK
    DCOCTL = CALDCO_8MHZ;
    BCSCTL2 |= DIVS_1;     //SMCLK = DCO/2 (4MHz) 
    BCSCTL3 |= LFXT1S_2;                   // Use VLO for ACLK
    //
    //    Initialize Ports
    //
    /*PORT 1:
    1.0     LED 1
    1.1     LED 2
    1.2     Button - connects to GND when closed
    1.3     Accelerometer Interrupt
    1.4     
    1.5     
    1.6     
    1.7     
    */
    P1DIR = BIT0+BIT1;
    P1OUT &= ~BIT2;
    P1IE  = BIT2;   //Enable Button Interrupt
    P1IES = BIT2;   //Interrupt on high-to-low transition
    P1REN = BIT2;   //enable resistor for button
    P1OUT = BIT2;   //make resistor a PULL-UP
    P1SEL = 0;      //NOTE: default value is NOT 0!
    P1IFG = 0;
    /*PORT 2
    2.0     
    2.1     Light Sensor power
    2.2     TEST
    2.3     
    2.4     
    2.5     
    2.6     SRDY
    2.7     Accelerometer SS
    */
    P2DIR = BIT1+BIT2+BIT3;
    P2SEL = 0;
    
    /*PORT 3
    3.0     ZNP CS
    3.1     SPI MOSI
    3.2     SPI MISO
    3.3     SPI SCLK
    3.4     Debug UART Tx
    3.5     Debug UART Rx
    3.6     MRDY
    3.7     ZNP Reset
    */
    P3DIR = BIT0+BIT6+BIT7;
    P3SEL = (BIT1+BIT2+BIT3+BIT4+BIT5);  //p3.1,2,3=USCI_B0; P3.4,5 = USCI_A0 TXD/RXD;
    P3OUT &= ~BIT7;                      //turn off radio
    
    /*PORT 4
    4.0     CFG 0
    4.1     CFG 1
    4.2     CC2530 programming DC
    4.3     CC2530 programming DD
    4.4     
    4.5     
    4.6     
    4.7     
    */
    P4DIR = BIT3+BIT4+BIT5+BIT6;
    P4SEL = 0;
    P4OUT &= ~BIT4;
    //
    //    Initialize UART debug console:
    //
    UCA0CTL1 |= UCSSEL_2;                     // SMCLK
    UCA0BR0 = 26; UCA0BR1 = 0;                // 4mHz smclk w/modulation for 9,600bps, table 15-5 
    UCA0MCTL = UCBRS_0 + +UCBRF_1 + UCOS16;   // Modulation UCBRSx=1, over sampling      
    UCA0CTL1 &= ~UCSWRST;                     // **Initialize USCI state machine**
    IE2 |= UCA0RXIE;                          // Enable USCI_A0 RX interrupt
    
    
    //  Stop Timer A:
    TACTL = MC_0; 
    
    clearLeds();
}

/**Initialize Timer*/
void Init_Timer(void)
{
    unsigned int vlo = calibrateVlo(); 
    signed int timerResult;
    timerIsr = &handleTimer;  
    
    timerResult = initTimer(4, WAKEUP_AFTER_TIMER);
    if (timerResult != 0)
    {
        printf("timerResult Error %i, STOPPING\r\n", timerResult);
        while (1);   
    }  
}

/** Send one byte via hardware UART. Required for printf() etc. in stdio.h */
int putchar(int c)
{	
    while (!(IFG2 & UCA0TXIFG));   // Wait for ready
    UCA0TXBUF = (unsigned char) (c & 0xFF); 
    return c;
}


/**
* Blocking Delay in Milliseconds
* delays by at least the specified number of milliseconds (ms)
* @pre TICKS_PER_MS set
* @param ms number of milliseconds to delay
*/
void delayMs(unsigned int ms)    
{
    unsigned int a, b;
    for (a = ms; a > 0; a--)         // outer loop takes 5 ck per round  
        for (b = TICKS_PER_MS; b > 0; b--)  // inner loop takes 5 ck per round 
            asm(" nop"); 
}

/** Blocking Delay in Microseconds
delays by at least the specified number of microseconds (us)
@pre TICKS_PER_US set
@param us number of microseconds to delay
*/
void delayUs(unsigned int us)
{   
    unsigned int a; 
    us *= TICKS_PER_US; 
    for (a = us; a > 0; a--)  // loop takes 5 ck per round 
        	asm(" nop"); 
}

/** Turns ON the specified LED. 
@param led the LED to turn on, must be 0 or 1.
@post The specified LED is turned on. 
@return 0 if success, -1 if invalid LED specified
*/
signed int setLed(unsigned char led)
{
    if (led > 1) 
        return -1;
    P1OUT |= (led) ? BIT1 : BIT0;
    return 0;
}

/** Turns OFF LEDs. 
@post LEDs are turned off. 
*/
void clearLeds()
{
    P1OUT &= ~(BIT0 + BIT1);
}

/** Toggles the specified LED. 
@param led the LED to toggle, must be 0 or 1.
@post The specified LED is toggled. 
@return 0 if success, -1 if invalid LED specified
*/
signed int toggleLed(unsigned char led)
{
    if (led > 1) 
        return -1;
    P1OUT ^= (1 << led);
    return 0;
}


#define TIMER_MAX_SECONDS 4
/** Configures timer.
@pre ACLK sourced from VLO
@pre VLO has been calibrated; number of VLO counts in one second is in vloFrequency.
@param seconds period of the timer. Maximum is 0xFFFF / vloFrequency; or about 4 since VLO varies between 9kHz - 15kHz. 
Use a prescaler on timer (e.g. set IDx bits in TACTL register) for longer times. 
Maximum prescaling of Timer A is divide by 8. Even longer times can be obtained by prescaling ACLK if this doesn't affect other system peripherals.
@param wakeOnTimer whether to wake the processor after the timer interrupt. Must be either NO_WAKEUP or WAKEUP_AFTER_TIMER.
@return 0 if success; -1 if illegal parameter or -2 if VLO not calibrated
*/
signed int initTimer(unsigned char seconds, unsigned char wakeOnTimer)
{  
    if (seconds > TIMER_MAX_SECONDS)
        return -1;
    if (vloFrequency == 0)
        return -2;
    if ((wakeOnTimer != NO_WAKEUP) && (wakeOnTimer != WAKEUP_AFTER_TIMER))
        return -3;
    wakeupFlags |= WAKEUP_AFTER_TIMER;
    CCTL0 = CCIE;                             // CCR0 interrupt enabled
    CCR0 = vloFrequency * (seconds);          // generate int
    TACTL = TASSEL_1 + MC_1;           // ACLK, upmode
    return 0;
}


void stopTimer()
{
    TACTL = MC_0; 
}

#pragma vector=TIMERA0_VECTOR
__interrupt void Timer_A (void)
{
    timerIsr();
    if (wakeupFlags & WAKEUP_AFTER_TIMER)    
    {
        HAL_WAKEUP();     
    }
}

/** Calibrate VLO. Once this is done, the VLO can be used semi-accurately for timers etc. 
Once calibrated, VLO is within ~2% of actual when using a 1% calibrated DCO frequency and temperature and supply voltage remain unchanged.
@return VLO frequency (number of VLO counts in 1sec), or -1 if out of range
@pre SMCLK is 4MHz
@pre MCLK is 8MHz
@pre ACLK sourced by VLO (BCSCTL3 = LFXT1S_2; in MSP430F2xxx)
@note Calibration is only as good as MCLK source. Obviously, if using the internal DCO (+/- 1%) then this value will only be as good as +/- 1%. YMMV.
@note On MSP430F248 or MSP430F22x2 or MSP430F22x4, must use TACCR2. On MSP430F20x2, must use TACCR0.
Check device-specific datasheet to see which module block has ACLK as a compare input.
For example, see page 23 of the MSP430F24x datasheet or page 17 of the MSP430F20x2 datasheet, or page 18 of the MSP430F22x4 datasheet.
@note If application will require accuracy over change in temperature or supply voltage, recommend calibrating VLO more often.
@post Timer A settings changed
@post ACLK divide by 8 bit cleared
*/

signed int calibrateVlo()
{
	unsigned int firstCapture;
	unsigned long counts;
    WDTCTL = WDTPW + WDTHOLD;             // Stop watchdog timer
    delayMs(1000);
    
    BCSCTL1 |= DIVA_3;                    // Divide ACLK by 8
    TACCTL2 = CM_1 + CCIS_1 + CAP;        // Capture on ACLK
    TACTL = TASSEL_2 + MC_2 + TACLR;      // Start TA, SMCLK(DCO), Continuous
    while ((TACCTL0 & CCIFG) == 0);       // Wait until capture
    
    TACCR2 = 0;                           // Ignore first capture
    TACCTL2 &= ~CCIFG;                    // Clear CCIFG
    
    while ((TACCTL2 & CCIFG) == 0);       // Wait for next capture
    firstCapture = TACCR2;   // Save first capture
    TACCTL2 &= ~CCIFG;                    // Clear CCIFG
    
    while ((TACCTL2 & CCIFG) ==0);        // Wait for next capture
    
    counts = (TACCR2 - firstCapture);        // # of VLO clocks in 8Mhz
    BCSCTL1 &= ~DIVA_3;                   // Clear ACLK/8 settings
    
    vloFrequency = ((unsigned int) (32000000l / counts));
    if ((vloFrequency > VLO_MIN) && (vloFrequency < VLO_MAX))
        return vloFrequency;
    else
        return -1;
}



static void out(char c) {
    *bf++ = c;
    }

static void outDgt(char dgt) {
	out(dgt+(dgt<10 ? '0' : (uc ? 'A' : 'a')-10));
	zs=1;
    }
	
static void divOut(unsigned int div) {
    unsigned char dgt=0;
	num &= 0xffff; // just for testing the code  with 32 bit ints
	while (num>=div) {
		num -= div;
		dgt++;
		}
	if (zs || dgt>0) 
		outDgt(dgt);
    }	

void tfp_printf(char *fmt, ...)
	{
	va_list va;
	char ch;
	char* p;
	
	va_start(va,fmt);
	
	while ((ch=*(fmt++))) {
		if (ch!='%') {
			putchar(ch);
			}
		else {
			char lz=0;
			char w=0;
			ch=*(fmt++);
			if (ch=='0') {
				ch=*(fmt++);
				lz=1;
				}
			if (ch>='0' && ch<='9') {
				w=0;
				while (ch>='0' && ch<='9') {
					w=(((w<<2)+w)<<1)+ch-'0';
					ch=*fmt++;
					}
				}
			bf=buf;
			p=bf;
			zs=0;
			switch (ch) {
				case 0: 
					goto abort;
				case 'u':
				case 'd':
                                case 'i':
					num=va_arg(va, unsigned int);
					if (((ch=='d')||(ch=='i')) && (int)num<0) {
						num = -(int)num;
						out('-');
						}
					divOut(10000);
					divOut(1000);
					divOut(100);
					divOut(10);
					outDgt(num);
					break;
				case 'x': 
				case 'X' : 
				    uc= ch=='X';
					num=va_arg(va, unsigned int);
					divOut(0x1000);
					divOut(0x100);
					divOut(0x10);
					outDgt(num);
					break;
				case 'c' : 
					out((char)(va_arg(va, int)));
					break;
				case 's' : 
					p=va_arg(va, char*);
					break;
				case '%' :
					out('%');
				default:
					break;
				}
			*bf=0;
			bf=p;
			while (*bf++ && w > 0)
				w--;
			while (w-- > 0) 
				putchar(lz ? '0' : ' ');
			while ((ch= *p++))
				putchar(ch);
			}
		}
	abort:;
	va_end(va);
	}
	
#ifdef Convert_to_lux
#include <math.h>
/**
Converts the raw ADC value to lux for the Osram SFH 5711 light sensor using pow() method in math.h.
Conversion:
- Convert ADC to millivolts: mV=(ADC/1024) * 2500  (10bit ADC = 1024counts; we're using the 2.5V reference)
- Convert millivolts to microAmps: uA = mV / 47 (since the design uses a 47kOhm load resistor)
- Convert microAmps to lux: lux = 10^(uA/10)
@param adc the raw reading from the Analog to Digital converter
@return the amount of light, in lux/10.
*/
unsigned int convertAdcToLux(unsigned int adc)
{
    const float multiplier =(2500.0/47.0)/1024.0;
    return ((unsigned int) pow(10, ((adc*multiplier)/10) - 1));
}
#endif
